#ifndef SIMULATION_H
#define SIMULATION_H

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <time.h>

#define MAXLEN 1024
#define MISSVAL -9.0
#define NUMTOL 1e-6
#define ANALYSIS_STATUS 1
#define MAXFILELEN 2001 
#define MAXIDLEN 20 //maximum length of ID
#define NITER_MAX 1e6



#ifdef __cplusplus
extern "C"
{
#endif

    struct KER_MAT {
        double **mat;   
        int nrow;
        int ncol;
        double **cenmat;
    };

    struct COV_MAT {
        double **phi; // GRM
        double **V; // eigen vectors
        double *D; // eigen values
    };

    struct STAT_MATS {
        double **KAT_Y; 
        double **prematY;
        double **prematG;
        double **pvals;
        int dim_premat;
    };

    struct DATA_STRUCT{
        int ntotal;
        int npheno;
        int nmarker;
    };

    struct QUANTS{ 
        long double T1;
        long double T2;
        long double T3;
        long double S1;
        long double S2;
        long double S3;
        long double R1; 
        long double R2;
        long double R3;
        long double R4;
        long double C1;
        long double C2;
        long double C3; 
        long double C4;
        long double C5;
        long double D1;
        long double D2;

        int is_computed;
    };


    struct PARAMS { 
        double pvalue;
        double Q;
        long double EQ;
        long double EQ2;
        long double EQ3;
        double loc;
        double sh;
        double sc;
    };


#include "gadev.h"
#include "nrutil.h"
#include "rand1.h"
#include "hashes.h"


    // My functions
#include "read.h"

    void get_eigendecomp(struct COV_MAT *cov_mats, struct KER_MAT* cmat);
    void writeBin_eigen(char *fileout, struct COV_MAT *cov_mats, int ntotal);
    double get_pval(struct KER_MAT *Y, struct KER_MAT *G, struct KER_MAT *covG, struct COV_MAT *cov_mats, int niter);

#ifdef __cplusplus
}
#endif

#endif
